@@ -1,6 +1,7 @@
 /*
     SPDX-FileCopyrightText: 2012-2013 Jan Grulich <jgrulich@redhat.com>
     SPDX-FileCopyrightText: 2013 Daniel Nicoletti <dantti12@gmail.com>
+    SPDX-FileCopyrightText: 2023 Volker Krause <vkrause@kde.org>
 
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
@@ -8,30 +9,30 @@
 #include "connectionsettings.h"
 #include "connectionsettings_p.h"
 
-#include "wiredsetting.h"
-#include "olpcmeshsetting.h"
-#include "wirelesssetting.h"
-#include "wirelesssecuritysetting.h"
-#include "security8021xsetting.h"
 #include "adslsetting.h"
 #include "bluetoothsetting.h"
 #include "bondsetting.h"
-#include "bridgesetting.h"
 #include "bridgeportsetting.h"
-#include "gsmsetting.h"
+#include "bridgesetting.h"
 #include "cdmasetting.h"
+#include "gsmsetting.h"
 #include "infinibandsetting.h"
 #include "iptunnelsetting.h"
 #include "ipv4setting.h"
 #include "ipv6setting.h"
-#include "pppsetting.h"
+#include "olpcmeshsetting.h"
 #include "pppoesetting.h"
+#include "pppsetting.h"
+#include "security8021xsetting.h"
 #include "serialsetting.h"
 #include "tunsetting.h"
 #include "vlansetting.h"
 #include "vpnsetting.h"
 #include "wimaxsetting.h"
+#include "wiredsetting.h"
 #include "wireguardsetting.h"
+#include "wirelesssecuritysetting.h"
+#include "wirelesssetting.h"
 
 #undef signals
 
@@ -39,17 +40,22 @@
 #define NM_SETTING_CONNECTION_AUTOCONNECT_RETRIES "autoconnect-retries"
 #endif
 
+#if !NM_CHECK_VERSION(1, 12, 0)
+#define NM_SETTING_CONNECTION_MDNS "mdns"
+#endif
+
 #if !NM_CHECK_VERSION(1, 16, 0)
 #define NM_SETTING_WIREGUARD_SETTING_NAME "wireguard"
 #endif
 
-#include "teamsetting.h"
 #include "genericsetting.h"
+#include "teamsetting.h"
 
 #include <QUuid>
 
-namespace NetworkManager {
-    extern int checkVersion(const int x, const int y, const int z);
+namespace NetworkManager
+{
+extern bool checkVersion(const int x, const int y, const int z);
 }
 
 NetworkManager::ConnectionSettingsPrivate::ConnectionSettingsPrivate(ConnectionSettings *q)
@@ -64,8 +70,10 @@ NetworkManager::ConnectionSettingsPrivate::ConnectionSettingsPrivate(ConnectionS
     , autoconnectSlaves(ConnectionSettings::SlavesDefault)
     , lldp(ConnectionSettings::LldpDefault)
     , metered(ConnectionSettings::MeteredUnknown)
+    , mdns(ConnectionSettings::MdnsDefault)
     , q_ptr(q)
-{ }
+{
+}
 
 void NetworkManager::ConnectionSettingsPrivate::addSetting(const NetworkManager::Setting::Ptr &setting)
 {
@@ -95,7 +103,7 @@ void NetworkManager::ConnectionSettingsPrivate::initSettings(NMBluetoothCapabili
     case ConnectionSettings::Bluetooth:
         addSetting(Setting::Ptr(new BluetoothSetting()));
         addSetting(Setting::Ptr(new Ipv4Setting()));
-        //addSetting(Setting::Ptr(new Ipv6Setting()));
+        // addSetting(Setting::Ptr(new Ipv6Setting()));
         if (bt_cap == NM_BT_CAPABILITY_DUN) {
             addSetting(Setting::Ptr(new GsmSetting()));
             addSetting(Setting::Ptr(new PppSetting()));
@@ -135,7 +143,7 @@ void NetworkManager::ConnectionSettingsPrivate::initSettings(NMBluetoothCapabili
         break;
     case ConnectionSettings::Pppoe:
         addSetting(Setting::Ptr(new Ipv4Setting()));
-        //addSetting(new Ipv6Setting()));
+        // addSetting(new Ipv6Setting()));
         addSetting(Setting::Ptr(new PppSetting()));
         addSetting(Setting::Ptr(new PppoeSetting()));
         addSetting(Setting::Ptr(new WiredSetting()));
@@ -261,7 +269,7 @@ void NetworkManager::ConnectionSettingsPrivate::initSettings(const NetworkManage
         break;
     case ConnectionSettings::Pppoe:
         addSetting(connectionSettings->setting(Setting::Ipv4));
-        //addSetting(Ipv6Setting(connectionSettings->setting(Setting::Ipv6));
+        // addSetting(Ipv6Setting(connectionSettings->setting(Setting::Ipv6));
         addSetting(connectionSettings->setting(Setting::Ppp));
         addSetting(connectionSettings->setting(Setting::Pppoe));
         addSetting(connectionSettings->setting(Setting::Wired));
@@ -448,7 +456,8 @@ QString NetworkManager::ConnectionSettings::createNewUuid()
 
 NetworkManager::ConnectionSettings::ConnectionSettings()
     : d_ptr(new ConnectionSettingsPrivate(this))
-{ }
+{
+}
 
 NetworkManager::ConnectionSettings::ConnectionSettings(NetworkManager::ConnectionSettings::ConnectionType type, NMBluetoothCapabilities bt_cap)
     : d_ptr(new ConnectionSettingsPrivate(this))
@@ -478,6 +487,7 @@ NetworkManager::ConnectionSettings::ConnectionSettings(const NetworkManager::Con
     setAutoconnectSlaves(other->autoconnectSlaves());
     setLldp(other->lldp());
     setMetered(other->metered());
+    setMdns(other->mdns());
     setStableId(other->stableId());
 
     d->initSettings(other);
@@ -509,12 +519,8 @@ void NetworkManager::ConnectionSettings::fromMap(const NMVariantMapMap &map)
     }
     if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_PERMISSIONS))) {
         const QStringList permissions = connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_PERMISSIONS)).toStringList();
-        for (const QString & permission : permissions) {
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-            const QStringList split = permission.split(QLatin1Char(':'), QString::KeepEmptyParts);
-#else
+        for (const QString &permission : permissions) {
             const QStringList split = permission.split(QLatin1Char(':'), Qt::KeepEmptyParts);
-#endif
             addToPermissions(split.at(1), split.at(2));
         }
     }
@@ -561,7 +567,8 @@ void NetworkManager::ConnectionSettings::fromMap(const NMVariantMapMap &map)
     }
 
     if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT_SLAVES))) {
-        setAutoconnectSlaves((NetworkManager::ConnectionSettings::AutoconnectSlaves)connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT_SLAVES)).toInt());
+        setAutoconnectSlaves(
+            (NetworkManager::ConnectionSettings::AutoconnectSlaves)connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT_SLAVES)).toInt());
     }
 
     if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_LLDP))) {
@@ -572,6 +579,10 @@ void NetworkManager::ConnectionSettings::fromMap(const NMVariantMapMap &map)
         setMetered((NetworkManager::ConnectionSettings::Metered)connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_METERED)).toInt());
     }
 
+    if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_MDNS))) {
+        setMdns((NetworkManager::ConnectionSettings::Mdns)connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_MDNS)).toInt());
+    }
+
     if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_STABLE_ID))) {
         setStableId(connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_STABLE_ID)).toString());
     }
@@ -587,6 +598,70 @@ void NetworkManager::ConnectionSettings::fromMap(const NMVariantMapMap &map)
     }
 }
 
+static struct {
+    const char *name;
+    NetworkManager::Security8021xSetting::EapMethod method;
+} constexpr const eap_methods[] = {
+    {"PEAP", NetworkManager::Security8021xSetting::EapMethodPeap},
+    {"PWD", NetworkManager::Security8021xSetting::EapMethodPwd},
+    {"TLS", NetworkManager::Security8021xSetting::EapMethodTls},
+    {"TTLS", NetworkManager::Security8021xSetting::EapMethodTtls},
+};
+
+static struct {
+    const char *name;
+    NetworkManager::Security8021xSetting::AuthMethod method;
+} constexpr const auth_methods[] = {
+    {"GTC", NetworkManager::Security8021xSetting::AuthMethodGtc},
+    {"MSCHAP", NetworkManager::Security8021xSetting::AuthMethodMschap},
+    {"MSCHAPV2", NetworkManager::Security8021xSetting::AuthMethodMschapv2},
+    {"PAP", NetworkManager::Security8021xSetting::AuthMethodPap},
+};
+
+void NetworkManager::ConnectionSettings::fromMeCard(const QVariantMap &map)
+{
+    const auto ssid = map.value(QLatin1String("S")).toString();
+    setId(ssid);
+
+    auto wifiSetting = setting(Setting::Wireless).dynamicCast<WirelessSetting>();
+    wifiSetting->setInitialized(true);
+    wifiSetting->setSsid(ssid.toUtf8());
+
+    auto wifiSecurity = setting(Setting::WirelessSecurity).dynamicCast<WirelessSecuritySetting>();
+    const auto securityType = map.value(QLatin1String("T")).toString();
+
+    if (!securityType.isEmpty() && securityType != QLatin1String("nopass")) {
+        wifiSecurity->setInitialized(true);
+    }
+
+    if (securityType == QLatin1String("WPA") || securityType == QLatin1String("WEP")) {
+        wifiSecurity->setKeyMgmt(NetworkManager::WirelessSecuritySetting::WpaPsk);
+        wifiSecurity->setPsk(map.value(QLatin1String("P")).toString());
+        wifiSecurity->setPskFlags(NetworkManager::Setting::AgentOwned);
+    } else if (securityType == QLatin1String("WPA2-EAP")) {
+        wifiSecurity->setKeyMgmt(NetworkManager::WirelessSecuritySetting::WpaEap);
+        auto sec8021x = setting(Setting::Security8021x).dynamicCast<Security8021xSetting>();
+        sec8021x->setAnonymousIdentity(map.value(QLatin1String("A")).toString());
+        sec8021x->setIdentity(map.value(QLatin1String("I")).toString());
+        sec8021x->setPassword(map.value(QLatin1String("P")).toString());
+
+        const auto eapMethod = map.value(QLatin1String("E")).toString();
+        for (const auto &method : eap_methods) {
+            if (eapMethod == QLatin1String(method.name)) {
+                sec8021x->setEapMethods({method.method});
+                break;
+            }
+        }
+        const auto phase2AuthMethod = map.value(QLatin1String("PH2")).toString();
+        for (const auto &method : auth_methods) {
+            if (phase2AuthMethod == QLatin1String(method.name)) {
+                sec8021x->setPhase2AuthMethod(method.method);
+                break;
+            }
+        }
+    }
+}
+
 NMVariantMapMap NetworkManager::ConnectionSettings::toMap() const
 {
     NMVariantMapMap result;
@@ -665,6 +740,10 @@ NMVariantMapMap NetworkManager::ConnectionSettings::toMap() const
     connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_LLDP), lldp());
     connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_METERED), metered());
 
+    if (mdns() != MdnsDefault) {
+        connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_MDNS), mdns());
+    }
+
     if (!stableId().isEmpty()) {
         connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_STABLE_ID), stableId());
     }
@@ -752,14 +831,14 @@ void NetworkManager::ConnectionSettings::addToPermissions(const QString &user, c
     d->permissions.insert(user, type);
 }
 
-void NetworkManager::ConnectionSettings::setPermissions(const QHash< QString, QString > &perm)
+void NetworkManager::ConnectionSettings::setPermissions(const QHash<QString, QString> &perm)
 {
     Q_D(ConnectionSettings);
 
     d->permissions = perm;
 }
 
-QHash< QString, QString > NetworkManager::ConnectionSettings::permissions() const
+QHash<QString, QString> NetworkManager::ConnectionSettings::permissions() const
 {
     Q_D(const ConnectionSettings);
 
@@ -955,6 +1034,20 @@ void NetworkManager::ConnectionSettings::setMetered(NetworkManager::ConnectionSe
     d->metered = metered;
 }
 
+NetworkManager::ConnectionSettings::Mdns NetworkManager::ConnectionSettings::mdns() const
+{
+    Q_D(const ConnectionSettings);
+
+    return d->mdns;
+}
+
+void NetworkManager::ConnectionSettings::setMdns(NetworkManager::ConnectionSettings::Mdns mdns)
+{
+    Q_D(ConnectionSettings);
+
+    d->mdns = mdns;
+}
+
 QString NetworkManager::ConnectionSettings::stableId() const
 {
     Q_D(const ConnectionSettings);
@@ -993,7 +1086,7 @@ NetworkManager::Setting::List NetworkManager::ConnectionSettings::settings() con
     return d->settings;
 }
 
-QDebug NetworkManager::operator <<(QDebug dbg, const NetworkManager::ConnectionSettings &setting)
+QDebug NetworkManager::operator<<(QDebug dbg, const NetworkManager::ConnectionSettings &setting)
 {
     dbg.nospace() << "CONNECTION SETTINGS\n";
     dbg.nospace() << "===================\n";
@@ -1016,6 +1109,7 @@ QDebug NetworkManager::operator <<(QDebug dbg, const NetworkManager::ConnectionS
     dbg.nospace() << NM_SETTING_CONNECTION_AUTOCONNECT_SLAVES << ": " << setting.autoconnectSlaves() << '\n';
     dbg.nospace() << NM_SETTING_CONNECTION_LLDP << ": " << setting.lldp() << '\n';
     dbg.nospace() << NM_SETTING_CONNECTION_METERED << ": " << setting.metered() << '\n';
+    dbg.nospace() << NM_SETTING_CONNECTION_MDNS << ": " << setting.mdns() << '\n';
     dbg.nospace() << NM_SETTING_CONNECTION_STABLE_ID << ": " << setting.stableId() << '\n';
     dbg.nospace() << "===================\n";
     const auto settingsList = setting.settings();
